<?php

define('FIDDLEMAIL_FILTER_MAIL', 1);
define('FIDDLEMAIL_FILTER_COMMENTS', 2);
define('FIDDLEMAIL_FILTER_REGISTRATION', 4);

define('FIDDLEMAIL_COMMENT_BLOCK', 1);
define('FIDDLEMAIL_COMMENT_SPAM', 2);

define('FIDDLEMAIL_API_URL', 'https://fiddlemail.com/api/v1/');

/**
 * FiddlemailInstall
 *
 * @author M. Burkard
 */

abstract class Fiddlemail {
	
	public static $logtable = 'fiddlemail_log';
	public static $wblisttable = 'fiddlemail_wblist';
	private static $show_error = false;
	private static $would_block_comment = false;
	private static $check_context = '';

	public static function onAdminMenu() {
		add_options_page(__('Fiddlemail', 'fiddlemail'), __('Fiddlemail', 'fiddlemail'), 'manage_options', 'fiddlemail-settings', array('Fiddlemail', 'onSettingsPage'));
		add_action('admin_init', array('FiddleMail', 'onRegisterSettings'));
	}
	
	public static function onAdminNotices() {
		$api_key = get_option('fm_api_key');
		if(!$api_key) {
			$class = 'notice notice-warning';
			$message = __('You have not %settingslink% for Fiddlemail, yet. You can get one for free on %fmlink%.', 'fiddlemail');
			print '<div class="' . $class . '"><p>' . str_replace(array('%fmlink%', '%settingslink%'), array(__('<a href="https://fiddlemail.com" target="_blank">fiddlemail.com</a>', 'fiddlemail'), __('<a href="options-general.php?page=fiddlemail-settings">set an API key</a>', 'fiddlemail')), esc_html($message)) . '</p></div>'; 
		}
	}
	
	public static function onRegisterSettings() {
		register_setting('fm_api_status', 'fm_api_key', array('sanitize_callback' => array('Fiddlemail', 'validateAPIKey')));
		register_setting('fm_api_settings_main', 'fm_filter', array('sanitize_callback' => array('Fiddlemail', 'validateFilterSetting')));
		register_setting('fm_api_settings_main', 'fm_comment_handling');
		register_setting('fm_api_settings_extended', 'fm_api_send_mail');
		register_setting('fm_api_settings_extended', 'fm_api_block_score', array('sanitize_callback' => array('Fiddlemail', 'validateScoreValue')));
		
		register_setting('fm_api_settings_extended', 'fm_use_spamhaus');
		register_setting('fm_api_settings_extended', 'fm_use_uribl');
		
		register_setting('fm_api_settings_wblist', 'fm_whitelist', array('sanitize_callback' => array('Fiddlemail', 'validateWhitelist')));
		register_setting('fm_api_settings_wblist', 'fm_blacklist', array('sanitize_callback' => array('Fiddlemail', 'validateBlacklist')));
		
		add_settings_section('fm_api_status', __('Your Fiddlemail API key', 'fiddlemail'), array('Fiddlemail', 'showApiStatus'), 'fm_api_status');
		add_settings_field('fm_api_key', __('Fiddlemail API Key', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_status', 'fm_api_status', array('id' => 'fm_api_key'));
		add_settings_section('fm_api_settings_main', __('API Settings', 'fiddlemail'), array('Fiddlemail', 'showApiSettings'), 'fm_api_settings_main');
		add_settings_field('fm_filter', __('Filter at', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_main', 'fm_api_settings_main', array('id' => 'fm_filter'));
		add_settings_field('fm_comment_handling', __('Comment handling'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_main', 'fm_api_settings_main', array('id' => 'fm_comment_handling'));
		add_settings_section('fm_api_settings_extended', __('Extended Filtering Settings', 'fiddlemail'), array('Fiddlemail', 'showExtendedSettings'), 'fm_api_settings_extended');
		add_settings_field('fm_api_send_mail', __('Send full address', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_extended', 'fm_api_settings_extended', array('id' => 'fm_api_send_mail'));
		add_settings_field('fm_api_block_score', __('Filter at score', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_extended', 'fm_api_settings_extended', array('id' => 'fm_api_block_score'));
		
		add_settings_field('fm_use_spamhaus', __('Block email domains listed in Spamhaus blacklist', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_extended', 'fm_api_settings_extended', array('id' => 'fm_use_spamhaus'));
		add_settings_field('fm_use_uribl', __('Block email domains listed in URIBL blacklist', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_extended', 'fm_api_settings_extended', array('id' => 'fm_use_uribl'));
		
		add_settings_section('fm_api_settings_wblist', __('Blacklist/Whitelist', 'fiddlemail'), array('Fiddlemail', 'showWBListSettings'), 'fm_api_settings_wblist');
		add_settings_field('fm_whitelist', __('Whitelisted domains', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_wblist', 'fm_api_settings_wblist', array('id' => 'fm_whitelist'));
		add_settings_field('fm_blacklist', __('Blacklisted domains', 'fiddlemail'), array('Fiddlemail', 'showSettingsField'), 'fm_api_settings_wblist', 'fm_api_settings_wblist', array('id' => 'fm_blacklist'));
		
		add_settings_section('fm_api_log', __('Filter log', 'fiddlemail'), array('Fiddlemail', 'showFilterLog'), 'fm_api_log');
	}
	
	public static function validateScoreValue($value) {
		if($value < 0 || $value > 100) {
			add_settings_error('fm_api_block_score', 'INVALID_SCORE_VALUE', __('Please enter a score value between 0 and 100.', 'fiddlemail'));
			return false;
		}
		return $value;
	}
	
	public static function validateFilterSetting($value) {
		if($value == FIDDLEMAIL_FILTER_MAIL || $value == FIDDLEMAIL_FILTER_COMMENTS || $value == FIDDLEMAIL_FILTER_REGISTRATION) {
			return $value;
		} else {
			add_settings_error('fm_filter', 'INVALID_FILTER_VALUE', __('Please choose a valid filtering option.', 'fiddlemail'));
			return false;
		}
	}
	
	public static function validateBlacklist($value) {
		return self::validateDomainList($value, 'b');
	}
	
	public static function validateWhitelist($value) {
		return self::validateDomainList($value, 'w');
	}
	
	/**
	 * @global wpdb $wpdb
	 * @param string $value
	 * @param string $type w|b
	 * @return boolean
	 */
	public static function validateDomainList($value, $type = 'b') {
		global $wpdb;
		
		$valids = array();
		$domains = explode("\n", $value);
		for($i = 0; $i < count($domains); $i++) {
			$domain = trim($domains[$i]);
			if(self::isValidDomain($domain)) {
				//error_log($domain . ' is valid domain.');
				$valids[] = $domain;
			} else {
				//error_log($domain . ' is INvalid domain, removed.');
			}
		}
		
		$wpdb->delete($wpdb->prefix . self::$wblisttable, array('type' => $type));
		
		for($i = 0; $i < count($valids); $i++) {
			$wpdb->insert($wpdb->prefix . self::$wblisttable, array(
				'domain' => $valids[$i],
				'type' => $type
			));
			//error_log($domain . ' was added to list type ' . $type);
		}

		return false;
	}
	
	public static function validateAPIKey($value) {
		if($value != '' && preg_match('/^[0-9a-zA-Z]+$/', $value)) {
			$api_url = FIDDLEMAIL_API_URL . $value . '/status/check';
			$response = wp_remote_get($api_url, array(
				'timeout' => 2
			));
			if(is_array($response)) {
				$response = json_decode($response['body'], true);
				if(!$response || !is_array($response)) {
					$error_code = 'INVALID_API_RESPONSE';
				} elseif(isset($response['error']) && $response['error'] == true) {
					if(isset($response['message'])) {
						$error_code = $response['message'];
					}
				} else {
					// seems ok
					return $value;
				}
			} else {
				$error_code = 'COULD_NOT_REACH_API';
			}
		} else {
			$error_code = 'INVALID_API_KEY';
		}
		add_settings_error('fm_api_key', $error_code, __('The provided API key is not valid.', 'fiddlemail'));
		return false;
	}
	
	/**
	 * @global wpdb $wpdb
	 * @param array $args
	 * @return void
	 */
	public static function showSettingsField($args) {
		global $wpdb;
		
		if(!isset($args['id'])) {
			return;
		}

		$value = get_option($args['id']);
		$api_key = get_option('fm_api_key');
		$score_key = get_option('fm_api_key_score');
		
		if($args['id'] == 'fm_api_key') {
			print '<input type="text" name="' . $args['id'] . '" id="' . $args['id'] . '" value="' . esc_attr($value) . '" size="50" />';
		} elseif($args['id'] == 'fm_filter') {
			print '<select name="' . $args['id'] . '" id="' . $args['id'] . '">';
			print '<option value="' . FIDDLEMAIL_FILTER_MAIL . '"' . ($value == FIDDLEMAIL_FILTER_MAIL ? ' selected' : '') . '>' . __('Filter all email-related inputs.', 'fiddlemail') . '</option>';
			print '<option value="' . FIDDLEMAIL_FILTER_COMMENTS . '"' . ($value == FIDDLEMAIL_FILTER_COMMENTS ? ' selected' : '') . '>' . __('Filter comments posted with disposable address.', 'fiddlemail') . '</option>';
			print '<option value="' . FIDDLEMAIL_FILTER_REGISTRATION . '"' . ($value == FIDDLEMAIL_FILTER_REGISTRATION ? ' selected' : '') . '>' . __('Filter registrations using disposable address.', 'fiddlemail') . '</option>';
			print '</select>';
		} elseif($args['id'] == 'fm_api_block_score') {
			print '<input type="text" name="' . $args['id'] . '" id="' . $args['id'] . '" value="' . esc_attr($value) . '"' . ($score_key != 1 ? ' disabled' : '') . ' />
				<p class="description">' . __('Enter the score value from 0 to 100 from which email addresses should be blocked. A value below 50 is not recommended and could lead to false positives.', 'fiddlemail') . '</p>';
		} elseif($args['id'] == 'fm_api_send_mail') {
			print '<input type="checkbox" name="' . $args['id'] . '" id="' . $args['id'] . '" value="1"' . ($value == 1 ? ' checked' : '') . ($score_key != 1 ? ' disabled' : '') . ' />
				<p class="description">' . __('If you want to use the score API you need to submit the full email address. By default only the domain part of the address is sent to the API.', 'fiddlemail') . '</p>
				<p class="description">' . __('<strong>Info:</strong> Fiddlemail does NOT store the submitted email address. It is only used for score calculation and discarded afterwards. Only the domain part is stored.', 'fiddlemail') . '</p>';
 		} elseif($args['id'] == 'fm_comment_handling') {
			print '<select name="' . $args['id'] . '" id="' . $args['id'] . '"><option value="' . FIDDLEMAIL_COMMENT_BLOCK. '"' . ($value == FIDDLEMAIL_COMMENT_BLOCK ? ' selected' : '') . '>' . __('Block comment', 'fiddlemail') . '</option><option value="' . FIDDLEMAIL_COMMENT_SPAM . '"' . ($value == FIDDLEMAIL_COMMENT_SPAM ? ' selected' : '') . '>' . __('Mark comment as spam', 'fiddlemail') . '</option></select>';
		} elseif($args['id'] == 'fm_use_spamhaus') {
			print '<input type="checkbox" name="' . $args['id'] . '" id="' . $args['id'] . '" value="1"' . ($value == 1 ? ' checked' : '') . ' />
				<p class="description">' . __('Please check the <a href="https://www.spamhaus.org/organization/dnsblusage/" target="_blank">Usage Terms</a> of Spamhaus DBL before you activate this feature!', 'fiddlemail') . '</p>';
		} elseif($args['id'] == 'fm_use_uribl') {
			print '<input type="checkbox" name="' . $args['id'] . '" id="' . $args['id'] . '" value="1"' . ($value == 1 ? ' checked' : '') . ' />
				<p class="description">' . __('Please check the <a href="http://uribl.com/about.shtml" target="_blank">Usage Terms</a> of URIBL before you activate this feature!', 'fiddlemail') . '</p>';
		} elseif($args['id'] == 'fm_whitelist' || $args['id'] == 'fm_blacklist') {
			// read in white and blacklist from database
			$qrystr = 'SELECT `domain` FROM ' . $wpdb->prefix . self::$wblisttable. ' WHERE `type` = \'' . ($args['id'] == 'fm_whitelist' ? 'w' : 'b') . '\' ORDER BY `domain` ASC';
			$results = $wpdb->get_results($qrystr, ARRAY_A);
			$domains = array();
			foreach($results as $domain) {
				$domains[] = $domain['domain'];
			}
			
			$value = implode("\n", $domains);
			print '<textarea name="' . $args['id'] . '" id="' . $args['id'] . '" cols="80" rows="12">' . esc_html($value) . '</textarea>';
			if($args['id'] == 'fm_whitelist') {
				print '<p class="description">' . __('Please enter one domain per line. Domains listed here will not be sent to the API but treated as allowed immediately.', 'fiddlemail') . '</p>';
			} else {
				print '<p class="description">' . __('Please enter one domain per line. Domains listed here will not be sent to the API but blocked immediately.', 'fiddlemail') . '</p>';
			}
		}
	}
	
	/**
	 * @param string $email
	 * @param boolean $force_check
	 * @global wpdb $wpdb
	 */
	public static function checkEmail($email, $force_check = false) {
		global $wpdb;
		
		$admin_email = get_bloginfo('admin_email');
		if($email == $admin_email) {
			return true; // dont check this
		}
		$current_user = wp_get_current_user();
		if(isset($current_user->user_email) && $current_user->ID > 0 && $current_user->user_email == $email) {
			return true;
		}
		
		if(!$force_check && self::$check_context == 'comment' && get_option('fm_comment_handling') == FIDDLEMAIL_COMMENT_SPAM) {
			return true;
		}
		
		list(, $email_domain) = explode('@', $email);
		$email_domain = trim($email_domain);
		if($email_domain == '') {
			return false;
		}
		
		$api_key = get_option('fm_api_key');
		if(!$api_key) {
			return true;
		}
		
		$score_key = get_option('fm_api_key_score');
		$block_score = intval(get_option('fm_api_block_score'));
		if(!$block_score) {
			$block_score = 75;
		}
		$send_full_mail = get_option('fm_api_send_mail');
		
		$check_context = self::$check_context;
		if(!$check_context) {
			$check_context = $_SERVER['REQUEST_URI'];
			if(strpos($check_context, '?') !== false) {
				$check_context = substr($check_context, 0, strpos($check_context, '?'));
			}
			$check_context = basename($check_context);
		}
		
		/* check black and white lists */
		$qrystr = $wpdb->prepare('SELECT `type` FROM ' . $wpdb->prefix . self::$wblisttable . ' WHERE `domain` = %s', $email_domain);
		$wblist = $wpdb->get_row($qrystr, ARRAY_A);
		if($wblist) {
			$wpdb->insert($wpdb->prefix . self::$logtable, array(
				'logtime' => current_time('mysql'),
				'checked_mail' => ($score_key == 1 && $send_full_mail == 1 ? $email : $email_domain),
				'check_context' => $check_context,
				'score' => ($wblist['type'] == 'w' ? 0 : 100),
				'result' => ($wblist['type'] == 'w' ? 'whitelist' : 'blacklist')
			));
			return ($wblist['type'] == 'w' ? true : false);
		}

		$qrystr = $wpdb->prepare('SELECT `checked_mail`, `score`, `result`, TIMESTAMPDIFF(HOUR, `logtime`, NOW()) as `passed_time` FROM ' . $wpdb->prefix . self::$logtable . ' WHERE `checked_mail` = %s AND `logtime` >= DATE_SUB(NOW(), INTERVAL 24 HOUR) ORDER BY `logtime` DESC', ($send_full_mail == true ? $email : $email_domain));
		$cache = $wpdb->get_row($qrystr, ARRAY_A);
		if($cache && substr($cache['result'], 0, 5) != 'error') {
			if($cache['result'] != 'ok' || $cache['passed_time'] <= 4) {
				//only use cache if result was block or passed time less than 5 hours on "ok"
				$wpdb->insert($wpdb->prefix . self::$logtable, array(
					'logtime' => current_time('mysql'),
					'checked_mail' => ($score_key == 1 && $send_full_mail == 1 ? $email : $email_domain),
					'check_context' => $check_context,
					'score' => $cache['score'],
					'result' => $cache['result'] . ' [cache]'
				));
				return ($cache['result'] == 'ok' ? true : false);
			}
		}
		
		$block_score = $block_score / 100;
		
		$api_options = array(
			'score' => ($score_key == 1 ? true : false),
			'extended' => false
		);
		
		$api_url = FIDDLEMAIL_API_URL . $api_key . '/email/' . urlencode(($score_key == 1 && $send_full_mail == 1 ? $email : $email_domain));
		$response = wp_remote_post($api_url, array(
			'body' => json_encode($api_options),
			'timeout' => 2
		));
		if(!is_array($response)) {
			usleep(500000);
			// second try
			$response = wp_remote_post($api_url, array(
				'body' => json_encode($api_options),
				'timeout' => 2
			));
		}
		$score = 0;
		$api_result = 'error [noresponse]';
		$return = true;
		
		if(is_array($response)) {
			$response = json_decode($response['body'], true);
			if(!$response || !is_array($response)) {
				$return = true;
				$api_result = 'error [emptyresponse]';
			} elseif(isset($response['error']) && $response['error'] == true) {
				$return = true; // something went wrong!
				$api_result = 'error [invalid]';
			} elseif(isset($response['suspected']) && $response['suspected'] == true) {
				self::$show_error = true;
				$return = false; // fake email!
				$api_result = 'suspect';
				if(isset($response['score'])) {
					$score = round($response['score'] * 100);
				}
			} elseif(isset($response['score']) && $response['score'] >= $block_score) {
				self::$show_error = true;
				$return = false; // fake email!
				$api_result = 'suspect';
				$score = round($response['score'] * 100);
			} else {
				$return = true;
				$api_result = 'ok';
				if(isset($response['score'])) {
					$score = round($response['score'] * 100);
				}
			}
		}
		
		if($return == true) {
			$use_spamhaus = get_option('fm_use_spamhaus');
			$use_uribl = get_option('fm_use_uribl');
			
			if($use_spamhaus == 1 || $use_uribl == 1) {
				//require_once FM_INC_PATH . '/libdns.inc.php';
				require_once FM_INC_PATH . '/netdns2/Net/DNS2.php';
				$prev_apiresult = $api_result;
				$api_result = '';
				if($use_spamhaus == 1) {
					$dns_servers = array('x.ns.spamhaus.org', '8.ns.spamhaus.org', '0.ns.spamhaus.org');
					$result = self::getARecord(array_rand(array_flip($dns_servers)), $email_domain . '.dbl.spamhaus.org');
					if($result && in_array($result, array('127.0.1.2', '127.0.1.4', '127.0.1.5', '127.0.1.6'), true)) {
						$return = false;
						$api_result = 'blocked:spamhaus';
					}
				}
				
				if($return == true && $use_uribl == 1) {
					$dns_servers = array('ff.uribl.com');
					$result = self::getARecord(array_rand(array_flip($dns_servers)), $email_domain . '.multi.uribl.com');
					if($result && in_array($result, array('127.0.0.2', '127.0.0.4'), true)) {
						$return = false;
						$api_result = 'blocked:uribl';
					}
				}
				if($api_result == '') {
					$api_result = $prev_apiresult;
				}
				if($return == false && $score_key == 1) {
					$score = 100;
				}
			}
		}
		
		$wpdb->insert($wpdb->prefix . self::$logtable, array(
			'logtime' => current_time('mysql'),
			'checked_mail' => ($score_key == 1 && $send_full_mail == 1 ? $email : $email_domain),
			'check_context' => $check_context,
			'score' => $score,
			'result' => $api_result
		));
		return $return; // invalid response
	}
	
	public static function setCommentContext($post_id) {
		self::$check_context = 'comment';
	}
	
	public static function checkComment($comment) {
		self::$check_context = 'comment';
		$ok = self::checkEmail($comment['comment_author_email'], true);
		if(!$ok) {
			if(get_option('fm_comment_handling') == FIDDLEMAIL_COMMENT_SPAM) {
				self::$would_block_comment = true;
				return $comment;
			} else {
				wp_die(__('This email address is not allowed to post comments here.', 'fiddlemail'));
			}
		} else {
			self::$would_block_comment = false;
			return $comment;
		}
	}
	
	public static function handleComment($approved, $comment) {
		if(get_option('fm_comment_handling') == FIDDLEMAIL_COMMENT_SPAM) {
			if(self::$would_block_comment == true) {
				return 'spam';
			} else {
				return $approved;
			}
		} else {
			return $approved;
		}
	}

	public static function checkRegistration($errors, $user_login, $email) {
		self::$check_context = 'registration';
		$ok = self::checkEmail($email);
		if(!$ok) {
			$errors->add('fiddlemail_error', __('This email address is not allowed to register.', 'fiddlemail'));
		}
		return $errors;
	}
	
	/**
	 * @param WP_Error $errors
	 */
	public static function addErrorString($errors) {
		if(self::$show_error == true) {
			$message = __('Please use a valid email address.', 'fiddlemail');
			
			if($errors instanceof WP_Error) {
				$errors->add('INVALID_EMAIL', $message);
			} else {
				$errors .= '<br />' . $message;
			}
			
			self::$show_error = false;
		}
		
		return $errors;
	}
	
	public static function showApiStatus() {
		$api_key = get_option('fm_api_key');
		print '<div class="fm-api-status">';
		if(!$api_key) {
			print '<p><strong>' . __('You do not have an API key set. Please set one to view the status.', 'fiddlemail') . '</strong></p>';
		} else {
			$api_url = FIDDLEMAIL_API_URL . $api_key . '/status/check';
			$response = wp_remote_get($api_url, array(
				'timeout' => 2
			));
			if(is_array($response)) {
				$response = json_decode($response['body'], true);
				if(!$response || !is_array($response)) {
					print '<p>' . __('Failed getting the status. Please check that your server can reach fiddlemail.com', 'fiddlemail') . '</p>';
				} elseif(isset($response['error']) && $response['error'] == true) {
					update_option('fm_api_key_score', 0);
					print '<p>' . __('API key seems to be invalid.', 'fiddlemail') . '</p>';
				} else {
					update_option('fm_api_key_score', (isset($response['score']) && $response['score'] == true ? 1 : 0));
					print '<h2>' . __('Key status for key', 'fiddlemail') . ' ' . $api_key . '</h2>';
					print '<ul>
						<li><strong>' . __('API plan', 'fiddlemail') . ':</strong> ' . __('Fiddlemail', 'fiddlemail') . ' ' . esc_html(ucwords($response['plan'])) . '</li>
						<li><strong>' . __('Key active', 'fiddlemail') . ':</strong> ' . ($response['active'] == 1 ? __('Yes', 'fiddlemail') : __('No', 'fiddlemail')) . '</li>
						<li><strong>' . __('Valid until', 'fiddlemail') . ':</strong> ' . ($response['valid_until'] ? $response['valid_until'] : __('Unlimited', 'fiddlemail')) . '</li>
						<li><strong>' . __('Score feature available', 'fiddlemail') . ':</strong> ' . (isset($response['score']) && $response['score'] == true ? __('Yes', 'fiddlemail') : __('No', 'fiddlemail')) . '</li>
						<li><strong>' . __('Ip slots used', 'fiddlemail') . ':</strong> ' . $response['ip_usage'] . ' / ' . $response['ip_slots'] . '</li>
						<li><strong>' . __('Queries last minute', 'fiddlemail') . ':</strong> ' . $response['limits']['minute']['usage'] . ' / ' . ($response['limits']['minute']['limit'] > 0 ? $response['limits']['minute']['limit'] : __('unlimited', 'fiddlemail')) . '</li>
						<li><strong>' . __('Queries this hour', 'fiddlemail') . ':</strong> ' . $response['limits']['hour']['usage'] . ' / ' . ($response['limits']['hour']['limit'] > 0 ? $response['limits']['hour']['limit'] : __('unlimited', 'fiddlemail')) . '</li>
						<li><strong>' . __('Queries today', 'fiddlemail') . ':</strong> ' . $response['limits']['day']['usage'] . ' / ' . ($response['limits']['day']['limit'] > 0 ? $response['limits']['day']['limit'] : __('unlimited', 'fiddlemail')) . '</li>
						<li><strong>' . __('Queries this month', 'fiddlemail') . ':</strong> ' . $response['limits']['month']['usage'] . ' / ' . ($response['limits']['month']['limit'] > 0 ? $response['limits']['month']['limit'] : __('unlimited', 'fiddlemail')) . '</li>
						</ul>';
				}
			} else {
				print '<p>' . __('Invalid response from API.', 'fiddlemail') . '</p>';
			}
		}
	}
	
	public static function showApiSettings() {
		//print '<p>This text comes from showApiSettings</p>';
	}
	
	public static function showExtendedSettings() {
		//print '<p>This text comes from showApiSettings</p>';
	}
	
	public static function showWBListSettings() {
		_e('You can add your own whitelisted or blacklisted domains here.', 'fiddlemail');
	}
	
	/**
	 * @global wpdb $wpdb
	 */
	public static function showFilterLog() {
		global $wpdb;

		$qrystr = 'SELECT * FROM ' . $wpdb->prefix . self::$logtable . ' WHERE 1 ORDER BY logtime DESC LIMIT 1000';
		$results = $wpdb->get_results($qrystr, ARRAY_A);
		print '<style type="text/css">
table.fm-log-table th, table.fm-log-table td { text-align:left;}
			</style>
<div class="fm-log" style="width:100%;height:400px;overflow-y:scroll;">
<table class="fm-log-table" style="width:100%">
<thead>
<tr>
	<th>#</th>
	<th>' . __('Time') . '</th>
	<th>' . __('Queried Address') . '</th>
	<th>' . __('Checked At') . '</th>
	<th>' . __('Score') . '</th>
	<th>' . __('Result') . '</th>
</tr>
</thead>
<tbody>
';
		if($results) {
			foreach($results as $row) {
				print '<tr>
					<td>' . esc_html($row['id']) . '</td>
					<td>' . esc_html($row['logtime']) . '</td>
					<td>' . esc_html($row['checked_mail']) . '</td>
					<td>' . esc_html($row['check_context']) . '</td>
					<td>' . esc_html($row['score']) . '</td>
					<td>' . esc_html($row['result']) . '</td>
				</tr>';
			}
		}
		print '</tbody>
			</table>
			</div>';
	}
	
	public static function onSettingsPage() {
		if (!current_user_can('manage_options')) {
			wp_die( __('You do not have sufficient permissions to access this page.', 'fiddlemail') );
		}
		$api_key = get_option('fm_api_key');
		?>
			<div class="wrap">
				<h2><?php _e('Fiddlemail Settings', 'fiddlemail'); ?></h2>
				<p><?php _e('Here you find the settings for the Fiddlemail API. For information about the service or registering for an API key visit <a href="https://fiddlemail.com" target="_blank">Fiddlemail.com</a>.', 'fiddlemail'); ?></p>

				<?php
						if(isset($_GET['tab']) && $_GET['tab'] != '' && $_GET['tab'] != 'status' && !$api_key) {
							print '<div class="notice notice-error"><p>' . esc_html__('Please enter your API key first.', 'fiddlemail') . '</p></div>'; 
							
							unset($_GET['tab']);
						}
				?>
				
				<h2 class="nav-tab-wrapper">
					<a href="options-general.php?page=fiddlemail-settings&tab=status" class="nav-tab<?php if(!isset($_GET['tab']) || $_GET['tab'] == '' || $_GET['tab'] == 'status') { print ' nav-tab-active'; } ?>"><?php _e('API Status', 'fiddlemail'); ?></a>
					<a href="options-general.php?page=fiddlemail-settings&tab=settings" class="nav-tab<?php if($_GET['tab'] == 'settings') { print ' nav-tab-active'; } ?>"><?php _e('Settings', 'fiddlemail'); ?></a>
					<a href="options-general.php?page=fiddlemail-settings&tab=extended" class="nav-tab<?php if($_GET['tab'] == 'extended') { print ' nav-tab-active'; } ?>"><?php _e('Extended Settings', 'fiddlemail'); ?></a>
					<a href="options-general.php?page=fiddlemail-settings&tab=wblist" class="nav-tab<?php if($_GET['tab'] == 'wblist') { print ' nav-tab-active'; } ?>"><?php _e('Blacklist/Whitelist', 'fiddlemail'); ?></a>
					<a href="options-general.php?page=fiddlemail-settings&tab=log" class="nav-tab<?php if($_GET['tab'] == 'log') { print ' nav-tab-active'; } ?>"><?php _e('Filter Log', 'fiddlemail'); ?></a>
				</h2>				
				
				
				<form name="fmsettings" method="post" action="options.php">
					<?php
						if(!isset($_GET['tab']) || $_GET['tab'] == '' || $_GET['tab'] == 'status' || !$api_key) {
							settings_fields('fm_api_status'); 
							do_settings_sections('fm_api_status');
						} elseif($_GET['tab'] == 'settings') {
							settings_fields('fm_api_settings_main'); 
							do_settings_sections('fm_api_settings_main');
						} elseif($_GET['tab'] == 'extended') {
							settings_fields('fm_api_settings_extended'); 
							do_settings_sections('fm_api_settings_extended');
						} elseif($_GET['tab'] == 'wblist') {
							settings_fields('fm_api_settings_wblist'); 
							do_settings_sections('fm_api_settings_wblist');
						} elseif($_GET['tab'] == 'log') {
							settings_fields('fm_api_log'); 
							do_settings_sections('fm_api_log');
						}
					?>
					<?php if(!isset($_GET['tab']) || $_GET['tab'] != 'log') {
						?><p class="submit"><button type="submit" name="submit" class="button-primary"><?php _e('Save', 'fiddlemail'); ?></button></p>
						<?php
					}
					?>
				</form>
			</div>
		<?php
	}

	/**
	 * @param string $dns_server server to query
	 * @param string $domain domain to look up
	 * @param int $timeout query timeout in seconds
	 * @return boolean|string (first) IP of A record or false on error
	 */
	public function getARecord($dns_server, $domain, $timeout = 5) {
		$arecord = false;
		
		if(!class_exists('Net_DNS2')) {
			return false;
		}
		/*if(!class_exists('DNSQuery')) {
			return false;
		}
		$query = new DNSQuery($dns_server, 53, $timeout, true, false, false);
		$result = $query->Query($domain, 'A');

		if($query->error) {
			error_log($query->lasterror);
			return false;
		}

		for($i = 0; $i < $result->count; $i++) {
			$a = $result->results[$i]->data;
			if($a) {
				$arecord = $a;
				break;
			}
		}*/
		
		$dns_ip = @gethostbyname($dns_server);
		if(!$dns_ip) {
			return false;
		}
		
		$dns = new Net_DNS2_Resolver(array('nameservers' => array($dns_ip)));
		try {
			$result = $dns->query($domain, 'A');
		} catch (Net_DNS2_Exception $ex) {
			error_log('DNS query failed: ' . $ex->getMessage());
		}
		
		foreach($result->answer as $a) {
			//error_log(var_export($a, true));
			$arecord = $a->address;
		}

		return $arecord;
	}
	
	private static function isValidDomain($domain) {
	    return (preg_match("/^([a-z\d](-*[a-z\d])*)(\.([a-z\d](-*[a-z\d])*))*$/i", $domain)
		        && preg_match("/^.{1,253}$/", $domain)
			    && preg_match("/^[^\.]{1,63}(\.[^\.]{1,63})*$/", $domain));
	}
}
